from typing import Callable
from django.core.cache import cache
from functools import wraps
import json
import hashlib


def cache_result(timeout: int = 60*60*24):
    
    def decorator(func: Callable):

        @wraps(func)
        def wrapper(*args, **kwargs):
            key = f"{func.__module__}.{func.__name__}:" + hashlib.sha256(
                json.dumps({"args": args, "kwargs": kwargs}).encode()
            ).hexdigest()

            if (result := cache.get(key)) is not None:
                return result

            result = func(*args, **kwargs)
            cache.set(key, result, timeout)
            return result
        
        return wrapper
    
    return decorator



import re
from django import template
from django.template.defaultfilters import stringfilter
from django.utils.safestring import mark_safe


register = template.Library()

URL_PATTERN = re.compile(r"\[([^\]]+)\]\((https://[^\s)]+)\)")


@register.filter(name="linkify")
@stringfilter
def linkify(value: str) -> str:
    def replace_url(match):
        text = match.group(1)
        url = match.group(2)
        return f'<a href="{url}" target="_blank">{text}</a>'

    result = URL_PATTERN.sub(replace_url, value)
    return mark_safe(result)



from .columns import Columns
from .rows import Rows
from .flex import FlexColumns, FlexRows
from .space import Space


__all__ = [
    "Columns",
    "Rows",
    "FlexColumns",
    "FlexRows",
    "Space"
]
